@@ -31,8 +31,7 @@ Zotero.Collection = function (params = {}) {
   this._childCollections = new Set();
   this._childItems = new Set();
 
-  Zotero.Utilities.assignProps(this, params, ['name', 'libraryID', 'parentID',
-  'parentKey', 'lastSync']);
+  Zotero.Utilities.Internal.assignProps(this, params, ['name', 'libraryID', 'parentID', 'parentKey']);
 };
 
 Zotero.extendClass(Zotero.DataObject, Zotero.Collection);
@@ -112,8 +111,8 @@ Zotero.Collection.prototype.getName = function () {
 
 
 /*
-    * Populate collection data from a database row
-    */
+ * Populate collection data from a database row
+ */
 Zotero.Collection.prototype.loadFromRow = function (row) {
   var primaryFields = this._ObjectsClass.primaryFields;
   for (let i = 0; i < primaryFields.length; i++) {
@@ -153,6 +152,7 @@ Zotero.Collection.prototype.loadFromRow = function (row) {
 
       // Boolean
       case 'synced':
+      case 'deleted':
       case 'hasChildCollections':
       case 'hasChildItems':
         val = !!val;
@@ -174,9 +174,15 @@ Zotero.Collection.prototype.loadFromRow = function (row) {
 };
 
 
-Zotero.Collection.prototype.hasChildCollections = function () {
+Zotero.Collection.prototype.hasChildCollections = function (includeTrashed) {
   this._requireData('childCollections');
-  return this._childCollections.size > 0;
+  if (!this._childCollections.size) {
+    return false;
+  }
+  if (includeTrashed) {
+    return this._childCollections.size > 0;
+  }
+  return !this.getChildCollections().every((c) => c.deleted);
 };
 
 Zotero.Collection.prototype.hasChildItems = function () {
@@ -186,31 +192,31 @@ Zotero.Collection.prototype.hasChildItems = function () {
 
 
 /**
-    * Returns subcollections of this collection
-    *
-    * @param {Boolean} [asIDs=false] Return as collectionIDs
-    * @return {Zotero.Collection[]|Integer[]}
-    */
+ * Returns subcollections of this collection
+ *
+ * @param {Boolean} [asIDs=false] Return as collectionIDs
+ * @return {Zotero.Collection[]|Integer[]}
+ */
 Zotero.Collection.prototype.getChildCollections = function (asIDs) {
   this._requireData('childCollections');
 
   // Return collectionIDs
   if (asIDs) {
-    return this._childCollections.values();
+    return [...this._childCollections.values()];
   }
 
   // Return Zotero.Collection objects
-  return Array.from(this._childCollections).map(id => this.ObjectsClass.get(id));
+  return Array.from(this._childCollections).map((id) => this.ObjectsClass.get(id));
 };
 
 
 /**
-    * Returns child items of this collection
-    *
-    * @param	{Boolean}	asIDs			Return as itemIDs
-    * @param	{Boolean}	includeDeleted	Include items in Trash
-    * @return {Zotero.Item[]|Integer[]} - Array of Zotero.Item instances or itemIDs
-    */
+ * Returns child items of this collection
+ *
+ * @param	{Boolean}	asIDs			Return as itemIDs
+ * @param	{Boolean}	includeDeleted	Include items in Trash
+ * @return {Zotero.Item[]|Integer[]} - Array of Zotero.Item instances or itemIDs
+ */
 Zotero.Collection.prototype.getChildItems = function (asIDs, includeDeleted) {
   this._requireData('childItems');
 
@@ -229,7 +235,7 @@ Zotero.Collection.prototype.getChildItems = function (asIDs, includeDeleted) {
 
   // Return itemIDs
   if (asIDs) {
-    return childItems.map(item => item.id);
+    return childItems.map((item) => item.id);
   }
 
   // Return Zotero.Item objects
@@ -300,7 +306,7 @@ Zotero.Collection.prototype._saveData = Zotero.Promise.coroutine(function* (env)
     } else
     {
       let sql = 'UPDATE collections SET ' +
-      env.sqlColumns.map(x => x + '=?').join(', ') + ' WHERE collectionID=?';
+      env.sqlColumns.map((x) => x + '=?').join(', ') + ' WHERE collectionID=?';
       env.sqlValues.push(collectionID ? { int: collectionID } : null);
       yield Zotero.DB.queryAsync(sql, env.sqlValues);
     }
@@ -328,6 +334,19 @@ Zotero.Collection.prototype._saveData = Zotero.Promise.coroutine(function* (env)
       }.bind(this));
     }
   }
+
+  if (this._changedData.deleted !== undefined) {
+    if (this._changedData.deleted) {
+      sql = "REPLACE INTO deletedCollections (collectionID) VALUES (?)";
+    } else
+    {
+      sql = "DELETE FROM deletedCollections WHERE collectionID=?";
+    }
+    yield Zotero.DB.queryAsync(sql, collectionID);
+
+    this._clearChanged('deleted');
+    this._markForReload('primaryData');
+  }
 });
 
 Zotero.Collection.prototype._finalizeSave = Zotero.Promise.coroutine(function* (env) {
@@ -362,23 +381,23 @@ Zotero.Collection.prototype._finalizeSave = Zotero.Promise.coroutine(function* (
 
 
 /**
-     * @param {Number} itemID
-     * @return {Promise}
-     */
+ * @param {Number} itemID
+ * @return {Promise}
+ */
 Zotero.Collection.prototype.addItem = function (itemID, options) {
   return this.addItems([itemID], options);
 };
 
 
 /**
-    * Add multiple items to the collection in batch
-    *
-    * Requires a transaction
-    * Does not require a separate save()
-    *
-    * @param {Number[]} itemIDs
-    * @return {Promise}
-    */
+ * Add multiple items to the collection in batch
+ *
+ * Requires a transaction
+ * Does not require a separate save()
+ *
+ * @param {Number[]} itemIDs
+ * @return {Promise}
+ */
 Zotero.Collection.prototype.addItems = Zotero.Promise.coroutine(function* (itemIDs, options = {}) {
   options.skipDateModifiedUpdate = true;
 
@@ -406,25 +425,25 @@ Zotero.Collection.prototype.addItems = Zotero.Promise.coroutine(function* (itemI
 });
 
 /**
-     * Remove a item from the collection. The item is not deleted from the library.
-     *
-     * Requires a transaction
-     * Does not require a separate save()
-     *
-     * @return {Promise}
-     */
-Zotero.Collection.prototype.removeItem = function (itemID) {
-  return this.removeItems([itemID]);
+ * Remove a item from the collection. The item is not deleted from the library.
+ *
+ * Requires a transaction
+ * Does not require a separate save()
+ *
+ * @return {Promise}
+ */
+Zotero.Collection.prototype.removeItem = function (itemID, options = {}) {
+  return this.removeItems([itemID], options);
 };
 
 
 /**
-    * Remove multiple items from the collection in batch.
-    * The items are not deleted from the library.
-    *
-    * Does not require a separate save()
-    */
-Zotero.Collection.prototype.removeItems = Zotero.Promise.coroutine(function* (itemIDs) {
+ * Remove multiple items from the collection in batch.
+ * The items are not deleted from the library.
+ *
+ * Does not require a separate save()
+ */
+Zotero.Collection.prototype.removeItems = Zotero.Promise.coroutine(function* (itemIDs, options = {}) {
   if (!itemIDs || !itemIDs.length) {
     return;
   }
@@ -443,17 +462,18 @@ Zotero.Collection.prototype.removeItems = Zotero.Promise.coroutine(function* (it
     let item = yield this.ChildObjects.getAsync(itemID);
     item.removeFromCollection(this.id);
     yield item.save({
-      skipDateModifiedUpdate: true });
+      skipDateModifiedUpdate: true,
+      skipEditCheck: options.skipEditCheck });
 
   }
 });
 
 
 /**
-     * Check if an item belongs to the collection
-     *
-     * @param {Zotero.Item|Number} item - Item or itemID
-     */
+ * Check if an item belongs to the collection
+ *
+ * @param {Zotero.Item|Number} item - Item or itemID
+ */
 Zotero.Collection.prototype.hasItem = function (item) {
   this._requireData('childItems');
   if (item instanceof Zotero.Item) {
@@ -475,14 +495,14 @@ Zotero.Collection.prototype.hasDescendent = function (type, id) {
 
 
 /**
-    * Compares this collection to another
-    *
-    * Returns a two-element array containing two objects with the differing values,
-    * or FALSE if no differences
-    *
-    * @param	{Zotero.Collection}	collection			Zotero.Collection to compare this item to
-    * @param	{Boolean}		includeMatches			Include all fields, even those that aren't different
-    */
+ * Compares this collection to another
+ *
+ * Returns a two-element array containing two objects with the differing values,
+ * or FALSE if no differences
+ *
+ * @param	{Zotero.Collection}	collection			Zotero.Collection to compare this item to
+ * @param	{Boolean}		includeMatches			Include all fields, even those that aren't different
+ */
 Zotero.Collection.prototype.diff = function (collection, includeMatches) {
   var diff = [];
   var thisData = this.serialize();
@@ -533,10 +553,10 @@ Zotero.Collection.prototype.diff = function (collection, includeMatches) {
 
 
 /**
-    * Returns an unsaved copy of the collection without id and key
-    *
-    * Doesn't duplicate subcollections or items, because the collection isn't saved
-    */
+ * Returns an unsaved copy of the collection without id and key
+ *
+ * Doesn't duplicate subcollections or items, because the collection isn't saved
+ */
 Zotero.Collection.prototype.clone = function (libraryID) {
   Zotero.debug('Cloning collection ' + this.id);
 
@@ -564,8 +584,8 @@ Zotero.Collection.prototype.clone = function (libraryID) {
 
 
 /**
-   * Deletes collection and all descendent collections (and optionally items)
-   **/
+* Deletes collection and all descendent collections (and optionally items)
+**/
 Zotero.Collection.prototype._eraseData = Zotero.Promise.coroutine(function* (env) {
   Zotero.DB.requireTransaction();
 
@@ -587,20 +607,25 @@ Zotero.Collection.prototype._eraseData = Zotero.Promise.coroutine(function* (env
           libraryID: c.libraryID,
           key: c.key };
 
+        // skipDeleteLog is normally added to notifierData in DataObject::_finalizeErase(),
+        // so we have to do it manually here
+        if (env.options && env.options.skipDeleteLog) {
+          env.notifierData[c.id].skipDeleteLog = true;
+        }
       }
     }
     // Descendent items
     else {
-        // Trash/delete items
-        if (env.options.deleteItems) {
-          del.push(descendents[i].id);
-        }
-
-        // If item isn't being removed or is just moving to the trash, mark for update
-        if (!env.options.deleteItems || libraryHasTrash) {
-          itemsToUpdate.push(descendents[i].id);
-        }
+      // Trash/delete items
+      if (env.options.deleteItems) {
+        del.push(descendents[i].id);
       }
+
+      // If item isn't being removed or is just moving to the trash, mark for update
+      if (!env.options.deleteItems || libraryHasTrash) {
+        itemsToUpdate.push(descendents[i].id);
+      }
+    }
   }
   if (del.length) {
     if (libraryHasTrash) {
@@ -608,16 +633,16 @@ Zotero.Collection.prototype._eraseData = Zotero.Promise.coroutine(function* (env
     }
     // If library doesn't have trash, just erase
     else {
-        Zotero.debug(Zotero.Libraries.getName(this.libraryID) + " library does not have trash. " +
-        this.ChildObjects._ZDO_Objects + " will be erased");
-        let options = {};
-        Object.assign(options, env.options);
-        options.tx = false;
-        for (let i = 0; i < del.length; i++) {
-          let obj = yield this.ChildObjects.getAsync(del[i]);
-          yield obj.erase(options);
-        }
+      Zotero.debug(Zotero.Libraries.getName(this.libraryID) + " library does not have trash. " +
+      this.ChildObjects._ZDO_Objects + " will be erased");
+      let options = {};
+      Object.assign(options, env.options);
+      options.tx = false;
+      for (let i = 0; i < del.length; i++) {
+        let obj = yield this.ChildObjects.getAsync(del[i]);
+        yield obj.erase(options);
       }
+    }
   }
 
   // Update child collection cache of parent collection
@@ -655,9 +680,9 @@ Zotero.Collection.prototype._eraseData = Zotero.Promise.coroutine(function* (env
   // Update collection cache for descendant items
   if (itemsToUpdate.length) {
     let deletedCollections = new Set(env.deletedObjectIDs);
-    itemsToUpdate.forEach(itemID => {
+    itemsToUpdate.forEach((itemID) => {
       let item = Zotero.Items.get(itemID);
-      item._collections = item._collections.filter(c => !deletedCollections.has(c));
+      item._collections = item._collections.filter((c) => !deletedCollections.has(c));
     });
   }
 });
@@ -695,10 +720,10 @@ Zotero.Collection.prototype.serialize = function (nested) {
 
 
 /**
-    * Populate the object's data from an API JSON data object
-    *
-    * If this object is identified (has an id or library/key), loadAllData() must have been called.
-    */
+ * Populate the object's data from an API JSON data object
+ *
+ * If this object is identified (has an id or library/key), loadAllData() must have been called.
+ */
 Zotero.Collection.prototype.fromJSON = function (json, options = {}) {
   if (options.strict) {
     for (let prop in json) {
@@ -708,6 +733,7 @@ Zotero.Collection.prototype.fromJSON = function (json, options = {}) {
         case 'name':
         case 'parentCollection':
         case 'relations':
+        case 'deleted':
           break;
 
         default:
@@ -725,6 +751,10 @@ Zotero.Collection.prototype.fromJSON = function (json, options = {}) {
   this.parentKey = json.parentCollection ? json.parentCollection : false;
 
   this.setRelations(json.relations || {});
+
+  if (json.deleted || this.deleted) {
+    this.deleted = !!json.deleted;
+  }
 };
 
 
@@ -745,15 +775,15 @@ Zotero.Collection.prototype.toJSON = function (options = {}) {
 
 
 /**
-    * Returns an array of descendent collections and items
-    *
-    * @param	{Boolean}	[nested=false]		Return multidimensional array with 'children'
-    *											nodes instead of flat array
-    * @param	{String}	[type]				'item', 'collection', or NULL for both
-    * @param	{Boolean}	[includeDeletedItems=false]		Include items in Trash
-    * @return	{Object[]} - An array of objects with 'id', 'key', 'type' ('item' or 'collection'),
-    *     'parent', and, if collection, 'name' and the nesting 'level'
-    */
+ * Returns an array of descendent collections and items
+ *
+ * @param	{Boolean}	[nested=false]		Return multidimensional array with 'children'
+ *											nodes instead of flat array
+ * @param	{String}	[type]				'item', 'collection', or NULL for both
+ * @param	{Boolean}	[includeDeletedItems=false]		Include items in Trash
+ * @return	{Object[]} - An array of objects with 'id', 'key', 'type' ('item' or 'collection'),
+ *     'parent', and, if collection, 'name' and the nesting 'level'
+ */
 Zotero.Collection.prototype.getDescendents = function (nested, type, includeDeletedItems, level) {
   if (!this.id) {
     throw new Error('Cannot be called on an unsaved item');
@@ -774,7 +804,7 @@ Zotero.Collection.prototype.getDescendents = function (nested, type, includeDele
   }
 
   var collections = Zotero.Collections.getByParent(this.id);
-  var children = collections.map(c => ({
+  var children = collections.map((c) => ({
     id: c.id,
     name: c.name,
     type: 0,
@@ -782,7 +812,7 @@ Zotero.Collection.prototype.getDescendents = function (nested, type, includeDele
 
   if (!type || type == 'item') {
     let items = this.getChildItems(false, includeDeletedItems);
-    children = children.concat(items.map(i => ({
+    children = children.concat(items.map((i) => ({
       id: i.id,
       name: null,
       type: 1,
@@ -843,20 +873,20 @@ Zotero.Collection.prototype.getDescendents = function (nested, type, includeDele
 
 
 /**
-    * Return a collection in the specified library equivalent to this collection
-    *
-    * @return {Promise<Zotero.Collection>}
-    */
+ * Return a collection in the specified library equivalent to this collection
+ *
+ * @return {Promise<Zotero.Collection>}
+ */
 Zotero.Collection.prototype.getLinkedCollection = function (libraryID, bidrectional) {
   return this._getLinkedObject(libraryID, bidrectional);
 };
 
 
 /**
-    * Add a linked-object relation pointing to the given collection
-    *
-    * Does not require a separate save()
-    */
+ * Add a linked-object relation pointing to the given collection
+ *
+ * Does not require a separate save()
+ */
 Zotero.Collection.prototype.addLinkedCollection = Zotero.Promise.coroutine(function* (collection) {
   return this._addLinkedObject(collection);
 });
@@ -879,8 +909,8 @@ Zotero.Collection.prototype._registerChildCollection = function (collectionID) {
 
 
 /**
-    * Remove a collection from the cached child collections list if loaded
-    */
+ * Remove a collection from the cached child collections list if loaded
+ */
 Zotero.Collection.prototype._unregisterChildCollection = function (collectionID) {
   if (this._loaded.childCollections) {
     this._childCollections.delete(collectionID);
@@ -889,8 +919,8 @@ Zotero.Collection.prototype._unregisterChildCollection = function (collectionID)
 
 
 /**
-    * Add an item to the cached child items list if loaded
-    */
+ * Add an item to the cached child items list if loaded
+ */
 Zotero.Collection.prototype._registerChildItem = function (itemID) {
   if (this._loaded.childItems) {
     let item = this.ChildObjects.get(itemID);
@@ -902,8 +932,8 @@ Zotero.Collection.prototype._registerChildItem = function (itemID) {
 
 
 /**
-    * Remove an item from the cached child items list if loaded
-    */
+ * Remove an item from the cached child items list if loaded
+ */
 Zotero.Collection.prototype._unregisterChildItem = function (itemID) {
   if (this._loaded.childItems) {
     this._childItems.delete(itemID);
