@@ -76,11 +76,9 @@ Zotero.ItemFields = new function () {
     var baseFields = yield Zotero.DB.columnQueryAsync(sql);
 
     for (let field of fields) {
-      let isBaseField = baseFields.includes(field.fieldID);
       let label = field.label || Zotero.Schema.globalSchemaLocale.fields[field.fieldName];
-      // If string not available, use the field name, except for some base fields that aren't
-      // used in the UI and therefore aren't localized
-      if (!label && !['number', 'type', 'medium'].includes(field.fieldName)) {
+      // If string not available, use the field name
+      if (!label) {
         Zotero.logError(`Localized string not available for field '${field.fieldName}'`);
         label = Zotero.Utilities.Internal.camelToTitleCase(field.fieldName);
       }
@@ -347,7 +345,11 @@ Zotero.ItemFields = new function () {
 
 
   this.isAutocompleteField = function (field) {
-    field = this.getName(field);
+    var fieldName = this.getName(field);
+    if (!fieldName) {
+      Zotero.logError(`Can't check autocomplete for invalid field '${field}'`);
+      return false;
+    }
 
     var autoCompleteFields = [
     'journalAbbreviation',
@@ -377,7 +379,7 @@ Zotero.ItemFields = new function () {
       autoCompleteFields = autoCompleteFields.concat(add);
     }
 
-    return autoCompleteFields.indexOf(field) != -1;
+    return autoCompleteFields.includes(fieldName);
   };
 
 
