@@ -44,11 +44,11 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-                   * Begin download process for individual file
-                   *
-                   * @param {Zotero.Sync.Storage.Request} request
-                   * @return {Promise<Zotero.Sync.Storage.Result>}
-                   */
+   * Begin download process for individual file
+   *
+   * @param {Zotero.Sync.Storage.Request} request
+   * @return {Promise<Zotero.Sync.Storage.Result>}
+   */
   downloadFile: Zotero.Promise.coroutine(function* (request) {
     var item = Zotero.Sync.Storage.Utilities.getItemFromRequest(request);
     if (!item) {
@@ -122,13 +122,13 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
         }
         // If not compressed, check hash, in case only timestamp changed
         else if (!requestData.compressed && (yield item.attachmentHash) == requestData.md5) {
-            Zotero.debug("File hash matches remote file -- skipping download of " +
-            item.libraryKey);
-            updateHash = true;
-          } else
-          {
-            return true;
-          }
+          Zotero.debug("File hash matches remote file -- skipping download of " +
+          item.libraryKey);
+          updateHash = true;
+        } else
+        {
+          return true;
+        }
 
         // Update local metadata and stop request, skipping file download
         yield OS.File.setDates(path, null, new Date(requestData.mtime));
@@ -154,6 +154,8 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
         if (status != 200) {
           if (status == 404) {
             Zotero.debug("Remote file not found for item " + item.libraryKey);
+            // Don't refresh item pane rows when nothing happened
+            request.skipProgressBarUpdate = true;
             deferred.resolve(new Zotero.Sync.Storage.Result());
             return;
           }
@@ -257,15 +259,15 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
   uploadFile: Zotero.Promise.coroutine(function* (request) {
     var item = Zotero.Sync.Storage.Utilities.getItemFromRequest(request);
-    var multipleFiles = yield Zotero.Attachments.hasMultipleFiles(item);
+    var isZipUpload = yield this._isZipUpload(item);
 
-    // If we got a quota error for this library, skip upload for all multi-file attachments
+    // If we got a quota error for this library, skip upload for all zipped attachments
     // and for single-file attachments that are bigger than the remaining space. This is cleared
     // in storageEngine for manual syncs.
     var remaining = Zotero.Sync.Storage.Local.storageRemainingForLibrary.get(item.libraryID);
     if (remaining !== undefined) {
       let skip = false;
-      if (multipleFiles) {
+      if (isZipUpload) {
         Zotero.debug("Skipping multi-file upload after quota error");
         skip = true;
       } else
@@ -284,23 +286,43 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
         }
       }
       if (skip) {
+        // Stop trying to upload files if there's very little storage remaining
+        if (request.engine && remaining < Zotero.Sync.Storage.Local.STORAGE_REMAINING_MINIMUM) {
+          Zotero.debug(`${remaining} MB remaining in storage -- skipping further uploads`);
+          request.engine.stop('upload');
+        }
         throw yield this._getQuotaError(item);
       }
     }
 
-    if (multipleFiles) {
+    if (isZipUpload) {
       let created = yield Zotero.Sync.Storage.Utilities.createUploadFile(request);
       if (!created) {
         return new Zotero.Sync.Storage.Result();
       }
     }
-    return this._processUploadFile(request);
+
+    try {
+      return yield this._processUploadFile(request);
+    }
+    catch (e) {
+      // Stop trying to upload files if we hit a quota error and there's very little space
+      // remaining. If there's more space, we keep going, because it might just be a big file.
+      if (request.engine && e.error == Zotero.Error.ERROR_ZFS_OVER_QUOTA) {
+        let remaining = Zotero.Sync.Storage.Local.storageRemainingForLibrary.get(item.libraryID);
+        if (remaining < Zotero.Sync.Storage.Local.STORAGE_REMAINING_MINIMUM) {
+          Zotero.debug(`${remaining} MB remaining in storage -- skipping further uploads`);
+          request.engine.stop('upload');
+        }
+      }
+      throw e;
+    }
   }),
 
 
   /**
-       * Remove all synced files from the server
-       */
+   * Remove all synced files from the server
+   */
   purgeDeletedStorageFiles: Zotero.Promise.coroutine(function* (libraryID) {
     if (libraryID != Zotero.Libraries.userLibraryID) return;
 
@@ -336,17 +358,17 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-      * Get authorization from API for uploading file
-      *
-      * @param {Zotero.Item} item
-      * @return {Object|String} - Object with upload params or 'exists'
-      */
+   * Get authorization from API for uploading file
+   *
+   * @param {Zotero.Item} item
+   * @return {Object|String} - Object with upload params or 'exists'
+   */
   _getFileUploadParameters: Zotero.Promise.coroutine(function* (item) {
     var funcName = "Zotero.Sync.Storage.ZFS._getFileUploadParameters()";
 
     var path = item.getFilePath();
     var filename = OS.Path.basename(path);
-    var zip = yield Zotero.Attachments.hasMultipleFiles(item);
+    var zip = yield this._isZipUpload(item);
     if (zip) {
       var uploadPath = OS.Path.join(Zotero.getTempDirectory().path, item.key + '.zip');
     } else
@@ -449,17 +471,17 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
       // If remote attachment exists but has no hash (which can happen for an old (pre-4.0?)
       // attachment with just an mtime, or after a storage purge), send again with If-None-Match
       else if (result == "ERROR_412_WITHOUT_VERSION") {
-          if (headers["If-None-Match"]) {
-            throw new Error("412 returned for request with If-None-Match");
-          }
-          delete headers["If-Match"];
-          headers["If-None-Match"] = "*";
-          storedHash = null;
-          Zotero.debug("Retrying with If-None-Match");
-        } else
-        {
-          break;
+        if (headers["If-None-Match"]) {
+          throw new Error("412 returned for request with If-None-Match");
         }
+        delete headers["If-Match"];
+        headers["If-None-Match"] = "*";
+        storedHash = null;
+        Zotero.debug("Retrying with If-None-Match");
+      } else
+      {
+        break;
+      }
     }
 
     try {
@@ -499,10 +521,10 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-       * Handle known errors from upload authorization request
-       *
-       * These must be included in successCodes in _getFileUploadParameters()
-       */
+   * Handle known errors from upload authorization request
+   *
+   * These must be included in successCodes in _getFileUploadParameters()
+   */
   _handleUploadAuthorizationFailure: Zotero.Promise.coroutine(function* (req, item) {
     //
     // These must be included in successCodes above.
@@ -520,105 +542,105 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
     }
     // This shouldn't happen, but if it does, mark item for upload and restart sync
     else if (req.status == 404) {
-        Zotero.logError(`Item ${item.libraryID}/${item.key} not found in upload authorization ` +
-        'request -- marking for upload');
-        yield Zotero.Sync.Data.Local.markObjectAsUnsynced(item);
+      Zotero.logError(`Item ${item.libraryID}/${item.key} not found in upload authorization ` +
+      'request -- marking for upload');
+      yield Zotero.Sync.Data.Local.markObjectAsUnsynced(item);
+      return new Zotero.Sync.Storage.Result({
+        syncRequired: true });
+
+    } else
+    if (req.status == 412) {
+      let version = req.getResponseHeader('Last-Modified-Version');
+      if (!version) {
+        return "ERROR_412_WITHOUT_VERSION";
+      }
+      if (version > item.version) {
+        // Mark object for redownloading, in case the library version is up to date and
+        // it's just the attachment item that somehow didn't get updated
+        yield Zotero.Sync.Data.Local.addObjectsToSyncQueue(
+        'item', item.libraryID, [item.key], true);
+
         return new Zotero.Sync.Storage.Result({
           syncRequired: true });
 
-      } else
-      if (req.status == 412) {
-        let version = req.getResponseHeader('Last-Modified-Version');
-        if (!version) {
-          return "ERROR_412_WITHOUT_VERSION";
-        }
-        if (version > item.version) {
-          // Mark object for redownloading, in case the library version is up to date and
-          // it's just the attachment item that somehow didn't get updated
-          yield Zotero.Sync.Data.Local.addObjectsToSyncQueue(
-          'item', item.libraryID, [item.key], true);
+      }
 
-          return new Zotero.Sync.Storage.Result({
-            syncRequired: true });
+      // Get updated item metadata
+      let library = Zotero.Libraries.get(item.libraryID);
+      let { json, error } = yield this.apiClient.downloadObjects(
+      library.libraryType,
+      library.libraryTypeID,
+      'item',
+      [item.key])[
+      0];
+      if (error) {
+        Zotero.logError(error);
+        throw new Error(Zotero.Sync.Storage.defaultError);
+      }
+      if (json.length > 1) {
+        throw new Error("More than one result for item lookup");
+      }
 
-        }
+      yield Zotero.Sync.Data.Local.saveCacheObjects('item', item.libraryID, json);
+      json = json[0];
 
-        // Get updated item metadata
-        let library = Zotero.Libraries.get(item.libraryID);
-        let json = yield this.apiClient.downloadObjects(
-        library.libraryType,
-        library.libraryTypeID,
-        'item',
-        [item.key])[
-        0];
-        if (!Array.isArray(json)) {
-          Zotero.logError(json);
-          throw new Error(Zotero.Sync.Storage.defaultError);
-        }
-        if (json.length > 1) {
-          throw new Error("More than one result for item lookup");
-        }
-
-        yield Zotero.Sync.Data.Local.saveCacheObjects('item', item.libraryID, json);
-        json = json[0];
-
-        if (json.data.version > item.version) {
-          // Mark object for redownloading, in case the library version is up to date and
-          // it's just the attachment item that somehow didn't get updated
-          yield Zotero.Sync.Data.Local.addObjectsToSyncQueue(
-          'item', item.libraryID, [item.key], true);
-
-          return new Zotero.Sync.Storage.Result({
-            syncRequired: true });
-
-        }
-
-        let fileHash = yield item.attachmentHash;
-        let fileModTime = yield item.attachmentModificationTime;
-
-        Zotero.debug("MD5");
-        Zotero.debug(json.data.md5);
-        Zotero.debug(fileHash);
-
-        if (json.data.md5 == fileHash) {
-          item.attachmentSyncedModificationTime = fileModTime;
-          item.attachmentSyncedHash = fileHash;
-          item.attachmentSyncState = "in_sync";
-          yield item.saveTx({ skipAll: true });
-
-          return new Zotero.Sync.Storage.Result();
-        }
-
-        item.attachmentSyncState = "in_conflict";
-        yield item.saveTx({ skipAll: true });
+      if (json.data.version > item.version) {
+        // Mark object for redownloading, in case the library version is up to date and
+        // it's just the attachment item that somehow didn't get updated
+        yield Zotero.Sync.Data.Local.addObjectsToSyncQueue(
+        'item', item.libraryID, [item.key], true);
 
         return new Zotero.Sync.Storage.Result({
-          fileSyncRequired: true });
+          syncRequired: true });
 
-      } else
-      if (req.status == 413) {
-        let retry = req.getResponseHeader('Retry-After');
-        if (retry) {
-          let minutes = Math.round(retry / 60);
-          throw new Zotero.Error(
-          Zotero.getString('sync.storage.error.zfs.tooManyQueuedUploads', minutes),
-          "ZFS_UPLOAD_QUEUE_LIMIT");
-
-        }
-
-        // Store the remaining space so that we can skip files bigger than that until the next
-        // manual sync
-        let usage = req.getResponseHeader('Zotero-Storage-Usage');
-        let quota = req.getResponseHeader('Zotero-Storage-Quota');
-        Zotero.Sync.Storage.Local.storageRemainingForLibrary.set(item.libraryID, quota - usage);
-
-        throw yield this._getQuotaError(item);
       }
+
+      let fileHash = yield item.attachmentHash;
+      let fileModTime = yield item.attachmentModificationTime;
+
+      Zotero.debug("MD5");
+      Zotero.debug(json.data.md5);
+      Zotero.debug(fileHash);
+
+      if (json.data.md5 == fileHash) {
+        item.attachmentSyncedModificationTime = fileModTime;
+        item.attachmentSyncedHash = fileHash;
+        item.attachmentSyncState = "in_sync";
+        yield item.saveTx({ skipAll: true });
+
+        return new Zotero.Sync.Storage.Result();
+      }
+
+      item.attachmentSyncState = "in_conflict";
+      yield item.saveTx({ skipAll: true });
+
+      return new Zotero.Sync.Storage.Result({
+        fileSyncRequired: true });
+
+    } else
+    if (req.status == 413) {
+      let retry = req.getResponseHeader('Retry-After');
+      if (retry) {
+        let minutes = Math.round(retry / 60);
+        throw new Zotero.Error(
+        Zotero.getString('sync.storage.error.zfs.tooManyQueuedUploads', minutes),
+        "ZFS_UPLOAD_QUEUE_LIMIT");
+
+      }
+
+      // Store the remaining space so that we can skip files bigger than that until the next
+      // manual sync. Values are in megabytes.
+      let usage = req.getResponseHeader('Zotero-Storage-Usage');
+      let quota = req.getResponseHeader('Zotero-Storage-Quota');
+      Zotero.Sync.Storage.Local.storageRemainingForLibrary.set(item.libraryID, quota - usage);
+
+      throw yield this._getQuotaError(item);
+    }
   }),
 
   /**
-       * Given parameters from authorization, upload file to S3
-       */
+   * Given parameters from authorization, upload file to S3
+   */
   _uploadFile: Zotero.Promise.coroutine(function* (request, item, params) {
     if (request.isFinished()) {
       Zotero.debug("Upload request " + request.name + " is no longer running after getting " +
@@ -716,8 +738,8 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-       * Post-upload file registration with API
-       */
+   * Post-upload file registration with API
+   */
   _onUploadComplete: Zotero.Promise.coroutine(function* (req, request, item, params) {
     var uploadKey = params.uploadKey;
 
@@ -787,10 +809,10 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-       * Update the local attachment item with the mtime and hash of the uploaded file and the
-       * library version returned by the upload request, and save a modified version of the item
-       * to the sync cache
-       */
+   * Update the local attachment item with the mtime and hash of the uploaded file and the
+   * library version returned by the upload request, and save a modified version of the item
+   * to the sync cache
+   */
   _updateItemFileInfo: Zotero.Promise.coroutine(function* (item, params) {
     // Mark as in-sync
     yield Zotero.DB.executeTransaction(function* () {
@@ -818,7 +840,7 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
     });
 
     try {
-      if (yield Zotero.Attachments.hasMultipleFiles(item)) {
+      if (yield this._isZipUpload(item)) {
         var file = Zotero.getTempDirectory();
         file.append(item.key + '.zip');
         yield OS.File.remove(file.path);
@@ -837,7 +859,7 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
     Zotero.debug("Upload of attachment " + item.key + " cancelled with status code " + status);
 
     try {
-      if (yield Zotero.Attachments.hasMultipleFiles(item)) {
+      if (yield this._isZipUpload(item)) {
         var file = Zotero.getTempDirectory();
         file.append(item.key + '.zip');
         file.remove(false);
@@ -850,7 +872,7 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   _getUploadFile: Zotero.Promise.coroutine(function* (item) {
-    if (yield Zotero.Attachments.hasMultipleFiles(item)) {
+    if (yield this._isZipUpload(item)) {
       var file = Zotero.getTempDirectory();
       var filename = item.key + '.zip';
       file.append(filename);
@@ -863,13 +885,13 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-       * Get attachment item metadata on storage server
-       *
-       * @param {Zotero.Item} item
-       * @param {Zotero.Sync.Storage.Request} request
-       * @return {Promise<Object>|false} - Promise for object with 'hash', 'filename', 'mtime',
-       *                                   'compressed', or false if item not found
-       */
+   * Get attachment item metadata on storage server
+   *
+   * @param {Zotero.Item} item
+   * @param {Zotero.Sync.Storage.Request} request
+   * @return {Promise<Object>|false} - Promise for object with 'hash', 'filename', 'mtime',
+   *                                   'compressed', or false if item not found
+   */
   _getStorageFileInfo: Zotero.Promise.coroutine(function* (item, request) {
     var funcName = "Zotero.Sync.Storage.ZFS._getStorageFileInfo()";
 
@@ -937,79 +959,79 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
 
 
   /**
-       * Upload the file to the server
-       *
-       * @param {Zotero.Sync.Storage.Request} request
-       * @return {Promise}
-       */
+   * Upload the file to the server
+   *
+   * @param {Zotero.Sync.Storage.Request} request
+   * @return {Promise}
+   */
   _processUploadFile: Zotero.Promise.coroutine(function* (request) {
     /*
-                                                                    updateSizeMultiplier(
-                                                                    	(100 - Zotero.Sync.Storage.compressionTracker.ratio) / 100
-                                                                    );
-                                                                    */
+    updateSizeMultiplier(
+    	(100 - Zotero.Sync.Storage.compressionTracker.ratio) / 100
+    );
+    */
 
     var item = Zotero.Sync.Storage.Utilities.getItemFromRequest(request);
 
 
     /*var info = yield this._getStorageFileInfo(item, request);
-                                                                          
-                                                                          if (request.isFinished()) {
-                                                                          	Zotero.debug("Upload request '" + request.name
-                                                                          		+ "' is no longer running after getting file info");
-                                                                          	return false;
-                                                                          }
-                                                                          
-                                                                          // Check for conflict
-                                                                          if (item.attachmentSyncState
-                                                                          		!= Zotero.Sync.Storage.Local.SYNC_STATE_FORCE_UPLOAD) {
-                                                                          	if (info) {
-                                                                          		// Local file time
-                                                                          		var fmtime = yield item.attachmentModificationTime;
-                                                                          		// Remote mod time
-                                                                          		var mtime = info.mtime;
-                                                                          		
-                                                                          		var useLocal = false;
-                                                                          		var same = !(yield Zotero.Sync.Storage.checkFileModTime(item, fmtime, mtime));
-                                                                          		
-                                                                          		// Ignore maxed-out 32-bit ints, from brief problem after switch to 32-bit servers
-                                                                          		if (!same && mtime == 2147483647) {
-                                                                          			Zotero.debug("Remote mod time is invalid -- uploading local file version");
-                                                                          			useLocal = true;
-                                                                          		}
-                                                                          		
-                                                                          		if (same) {
-                                                                          			yield Zotero.DB.executeTransaction(function* () {
-                                                                          				yield Zotero.Sync.Storage.setSyncedModificationTime(item.id, fmtime);
-                                                                          				yield Zotero.Sync.Storage.setSyncState(
-                                                                          					item.id, Zotero.Sync.Storage.Local.SYNC_STATE_IN_SYNC
-                                                                          				);
-                                                                          			});
-                                                                          			return {
-                                                                          				localChanges: true,
-                                                                          				remoteChanges: false
-                                                                          			};
-                                                                          		}
-                                                                          		
-                                                                          		let smtime = yield Zotero.Sync.Storage.getSyncedModificationTime(item.id);
-                                                                          		if (!useLocal && smtime != mtime) {
-                                                                          			Zotero.debug("Conflict -- last synced file mod time "
-                                                                          				+ "does not match time on storage server"
-                                                                          				+ " (" + smtime + " != " + mtime + ")");
-                                                                          			return {
-                                                                          				localChanges: false,
-                                                                          				remoteChanges: false,
-                                                                          				conflict: {
-                                                                          					local: { modTime: fmtime },
-                                                                          					remote: { modTime: mtime }
-                                                                          				}
-                                                                          			};
-                                                                          		}
-                                                                          	}
-                                                                          	else {
-                                                                          		Zotero.debug("Remote file not found for item " + item.libraryKey);
-                                                                          	}
-                                                                          }*/
+    
+    if (request.isFinished()) {
+    	Zotero.debug("Upload request '" + request.name
+    		+ "' is no longer running after getting file info");
+    	return false;
+    }
+    
+    // Check for conflict
+    if (item.attachmentSyncState
+    		!= Zotero.Sync.Storage.Local.SYNC_STATE_FORCE_UPLOAD) {
+    	if (info) {
+    		// Local file time
+    		var fmtime = yield item.attachmentModificationTime;
+    		// Remote mod time
+    		var mtime = info.mtime;
+    		
+    		var useLocal = false;
+    		var same = !(yield Zotero.Sync.Storage.checkFileModTime(item, fmtime, mtime));
+    		
+    		// Ignore maxed-out 32-bit ints, from brief problem after switch to 32-bit servers
+    		if (!same && mtime == 2147483647) {
+    			Zotero.debug("Remote mod time is invalid -- uploading local file version");
+    			useLocal = true;
+    		}
+    		
+    		if (same) {
+    			yield Zotero.DB.executeTransaction(function* () {
+    				yield Zotero.Sync.Storage.setSyncedModificationTime(item.id, fmtime);
+    				yield Zotero.Sync.Storage.setSyncState(
+    					item.id, Zotero.Sync.Storage.Local.SYNC_STATE_IN_SYNC
+    				);
+    			});
+    			return {
+    				localChanges: true,
+    				remoteChanges: false
+    			};
+    		}
+    		
+    		let smtime = yield Zotero.Sync.Storage.getSyncedModificationTime(item.id);
+    		if (!useLocal && smtime != mtime) {
+    			Zotero.debug("Conflict -- last synced file mod time "
+    				+ "does not match time on storage server"
+    				+ " (" + smtime + " != " + mtime + ")");
+    			return {
+    				localChanges: false,
+    				remoteChanges: false,
+    				conflict: {
+    					local: { modTime: fmtime },
+    					remote: { modTime: mtime }
+    				}
+    			};
+    		}
+    	}
+    	else {
+    		Zotero.debug("Remote file not found for item " + item.libraryKey);
+    	}
+    }*/
 
     var result = yield this._getFileUploadParameters(item);
     if (result.exists) {
@@ -1026,6 +1048,12 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
   }),
 
 
+  _isZipUpload: async function (item) {
+    return item.isImportedAttachment() && item.attachmentContentType.startsWith('text/') ||
+    Zotero.Attachments.hasMultipleFiles(item);
+  },
+
+
   _getQuotaError: async function (item) {
     var text,buttonText = null,buttonCallback;
     var libraryType = item.library.libraryType;
@@ -1038,15 +1066,15 @@ Zotero.Sync.Storage.Mode.ZFS.prototype = {
     }
     // Personal file
     else {
-        text = Zotero.getString('sync.storage.error.zfs.personalQuotaReached1') + "\n\n" +
-        Zotero.getString('sync.storage.error.zfs.personalQuotaReached2');
-        buttonText = Zotero.getString('sync.storage.openAccountSettings');
-        buttonCallback = function () {
-          let url = "https://www.zotero.org/settings/storage";
-          let win = Services.wm.getMostRecentWindow("navigator:browser");
-          win.ZoteroPane.loadURI(url, { metaKey: true, ctrlKey: true, shiftKey: true });
-        };
-      }
+      text = Zotero.getString('sync.storage.error.zfs.personalQuotaReached1') + "\n\n" +
+      Zotero.getString('sync.storage.error.zfs.personalQuotaReached2');
+      buttonText = Zotero.getString('sync.storage.openAccountSettings');
+      buttonCallback = function () {
+        let url = "https://www.zotero.org/settings/storage";
+        let win = Services.wm.getMostRecentWindow("navigator:browser");
+        win.ZoteroPane.loadURI(url, { metaKey: true, ctrlKey: true, shiftKey: true });
+      };
+    }
 
     var filename = item.attachmentFilename;
     var fileSize = (await OS.File.stat(item.getFilePath())).size;
