@@ -853,13 +853,21 @@ Zotero.Utilities.Internal = {
     var delay;
     var totalTime = 0;
     var last = false;
+    intervals = intervals.slice();
     while (true) {
       let interval = intervals.shift();
       if (interval) {
         delay = interval;
       }
 
+      // Be safe
+      if (!delay) {
+        Zotero.logError(`Incorrect delay ${delay} -- stopping`);
+        yield Zotero.Promise.resolve(false);
+      }
+
       if (maxTime && totalTime + delay > maxTime) {
+        Zotero.debug(`Total delay time exceeds ${maxTime} -- stopping`);
         yield Zotero.Promise.resolve(false);
       }
 
@@ -1263,14 +1271,9 @@ Zotero.Utilities.Internal = {
         return true;
       }
 
-      // Skip for now, since the mappings to Place will be changed
-      // https://github.com/citation-style-language/zotero-bits/issues/6
-      if (key == 'event-place' || key == 'publisher-place') {
-        return true;
-      }
-
       // Fields
       let possibleFields = fieldNames.get(key);
+
       // No valid fields
       if (possibleFields) {
         let added = false;
@@ -1561,6 +1564,18 @@ Zotero.Utilities.Internal = {
   },
 
 
+  /**
+   * Adds a localized colon to a string (which is usually just a colon, but, e.g., in French
+   * there's a space before it)
+   *
+   * @param {String}
+   * @return {String}
+   */
+  stringWithColon: function (str) {
+    return Zotero.getString('punctuation.colon.withString', str);
+  },
+
+
   resolveLocale: function (locale, locales) {
     // If the locale exists as-is, use it
     if (locales.includes(locale)) {
